@@ -24,7 +24,8 @@ Setups a cross-compilation environment for Microsoft Windows operating systems (
 # Require python 3.3 for shutil.which
 import shutil
 import subprocess
-import os
+import glob
+import os.path
 import sys
 
 install_datadir = os.path.join(os.path.abspath('@DATADIR@'), 'share')
@@ -89,8 +90,28 @@ def language_list():
             installed_languages.append(name)
     return (installed_languages, uninstalled_languages)
 
-def prepare():
-    pass
+def prepare(prefix):
+    '''
+    Prepare the environment.
+    Note that copying these libs is unnecessary for building, since the
+    system can find these at build time. But when moving the prefix to a
+    Windows machine, if ever we linked against these dll and they are
+    absent, the executable won't run.
+    '''
+    try:
+        env_bin = os.path.join(prefix, 'bin')
+        os.makedirs(env_bin, exist_ok = True)
+    except PermissionError:
+        sys.stderr.write('"{}" cannot be created. Please verify your permissions. Aborting.\n'.format(env_path))
+        return False
+    gcc_libs = subprocess.check_output(['x86_64-w64-mingw32-gcc', '-print-file-name='], universal_newlines=True)
+    for dll in glob.glob(gcc_libs.strip() + '/*.dll'):
+        try:
+            os.symlink(dll, os.path.join(os.path.join(env_bin, os.path.basename(dll))))
+        except OSError:
+            # A failed symlink is not necessarily a no-go. Let's just output a warning.
+            sys.stderr.write('Warning: crossroad failed to symlink {} in {}.\n'.format(dll, env_bin))
+    return True
 
 def crossroad_install(*packages:list, src:bool = False):
     '''
@@ -102,7 +123,7 @@ def crossroad_install(*packages:list, src:bool = False):
         sys.exit(os.EX_USAGE)
 
     command = [os.path.join(install_datadir, 'crossroad/scripts/crossroad-mingw-install.py'),
-               '-r', 'openSUSE_12.1', '-p', 'windows:mingw:win64', '--deps']
+               '-r', 'openSUSE_13.1', '-p', 'windows:mingw:win64', '--deps']
     if src:
         command += ['--src']
     command += list(packages)
@@ -117,7 +138,7 @@ def crossroad_list_files(*packages, src:bool = False):
         sys.exit(os.EX_USAGE)
 
     command = [os.path.join(install_datadir, 'crossroad/scripts/crossroad-mingw-install.py'),
-               '-r', 'openSUSE_12.1', '-p', 'windows:mingw:win64', '--list-files']
+               '-r', 'openSUSE_13.1', '-p', 'windows:mingw:win64', '--list-files']
     if src:
         command += ['--src']
     command += packages
@@ -132,7 +153,7 @@ def crossroad_info(*packages, src:bool = False):
         sys.exit(os.EX_USAGE)
 
     command = [os.path.join(install_datadir, 'crossroad/scripts/crossroad-mingw-install.py'),
-               '-r', 'openSUSE_12.1', '-p', 'windows:mingw:win64', '--info']
+               '-r', 'openSUSE_13.1', '-p', 'windows:mingw:win64', '--info']
     if src:
         command += ['--src']
     command += list(packages)
@@ -147,9 +168,27 @@ def crossroad_uninstall(*packages, src:bool = False):
         sys.exit(os.EX_USAGE)
 
     command = [os.path.join(install_datadir, 'crossroad/scripts/crossroad-mingw-install.py'),
-               '-r', 'openSUSE_12.1', '-p', 'windows:mingw:win64', '--uninstall']
+               '-r', 'openSUSE_13.1', '-p', 'windows:mingw:win64', '--uninstall']
     if src:
         command += ['--src']
     command += list(packages)
     subprocess.call(command, shell=False)
 
+def crossroad_search(*keywords, src:bool = False, search_files:bool = False):
+    '''
+    Search keywords in package names.
+    If --search-files is also set, also search in files.
+    '''
+    if len(keywords) == 0:
+        sys.stderr.write('Please provide at least one package name.\n')
+        sys.exit(os.EX_USAGE)
+
+    command = [os.path.join(install_datadir, 'crossroad/scripts/crossroad-mingw-install.py'),
+               '-r', 'openSUSE_13.1', '-p', 'windows:mingw:win64', '--search']
+    if src:
+        command += ['--src']
+    if search_files:
+        command += ['--list-files']
+    command += list(keywords)
+    subprocess.call(command, shell=False)
+
