@@ -7,6 +7,7 @@
 # compliance with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 
 from urllib.request import urlretrieve, urlopen
+import fnmatch
 import logging
 import os.path
 import sys
@@ -121,6 +122,12 @@ def OpenRepository(repositoryLocation):
         try:
           os.makedirs(dir)
         except OSError: pass
+      # Cleaning old files first.
+      for f in os.listdir(_repositoryCacheDirectory):
+        if f[-14:] == '-filelists.xml' and f != os.path.splitext(os.path.basename(filelist_url))[0]:
+            os.unlink(os.path.join(_repositoryCacheDirectory, f))
+        if f[-12:] == '-primary.xml' and f != os.path.splitext(os.path.basename(primary_url))[0]:
+            os.unlink(os.path.join(_repositoryCacheDirectory, f))
       # Download repository metadata (only if not already in cache)
       primary_filename = os.path.join(_repositoryCacheDirectory, os.path.splitext(os.path.basename(primary_url))[0])
       if not os.path.exists(primary_filename):
@@ -190,8 +197,31 @@ def OpenRepository(repositoryLocation):
         {'type': f.get('type', default='file'), 'path': f.text} for f in p.findall('{%s}file'%xmlns)
     ] for p in elements.findall('{%s}package'%xmlns) if p.get('arch') == 'src'}
 
-def _findPackage(packageName, srcpkg=False):
-  filter_func = lambda p: (p['name'] == packageName or p['filename'] == packageName) and p['arch'] == ('src' if srcpkg else 'noarch')
+def search_packages(keyword, srcpkg = False, search_files = False):
+  # Just in case the user was looking for a specific rpm file,
+  # I trim out the filename parts and keep the main naming.
+  keyword = re.sub('^mingw(32|64)-', '', packageBaseName(keyword.lower()))
+  packages = []
+  if search_files:
+      if srcpkg:
+          filelists = _package_src_filelists
+      else:
+          filelists = _package_filelists
+      filter_func = lambda p: \
+          len([f['path'] for f in filelists[p] if os.path.basename(f['path']).lower().find(keyword) != -1]) > 0
+      packages = sorted([p for p in filelists if filter_func(p)])
+  else:
+      filter_func = lambda p: \
+         re.sub('^mingw(32|64)-', '', p['name'].lower()).find(keyword) != -1 \
+         and p['arch'] == ('src' if srcpkg else 'noarch')
+      packages = sorted([p['name'] for p in _packages if filter_func(p)])
+  return packages
+
+def _findPackage(packageName, project, srcpkg=False):
+  filter_func = lambda p: \
+    ((p['name'] == 'mingw64-' + packageName if project == 'windows:mingw:win64' else False)
+     or p['name'] == 'mingw32-' + packageName or p['name'] == packageName or p['filename'] == packageName) \
+    and p['arch'] == ('src' if srcpkg else 'noarch')
   sort_func = lambda p: p['buildtime']
   packages = sorted([p for p in _packages if filter_func(p)], key=sort_func, reverse=True)
   if len(packages) == 0:
@@ -214,11 +244,13 @@ def _checkPackageRequirements(package, packageNames):
         allProviders.add(providers.pop())
   return allProviders
 
-def packagesDownload(packageNames, withDependencies = False, srcpkg = False, nocache = False):
-  from fnmatch import fnmatchcase
+def packageBaseName(rpm):
+    return re.sub(r'-([0-9]|\.)+-[0-9]\.[0-9].(src|noarch)\.(rpm|cpio)$', '', rpm)
+
+def packagesDownload(packageNames, project, withDependencies = False, srcpkg = False, nocache = False):
   packageNames_new = {pn for pn in packageNames if pn.endswith('.rpm')}
   for packageName in packageNames - packageNames_new:
-    matchedpackages = {p['name'] for p in _packages if fnmatchcase(p['name'].replace('mingw32-', '').replace('mingw64-', ''), packageName) and p['arch'] == ('src' if srcpkg else 'noarch')}
+    matchedpackages = {p['name'] for p in _packages if fnmatch.fnmatchcase(p['name'].replace('mingw32-', '').replace('mingw64-', ''), packageName) and p['arch'] == ('src' if srcpkg else 'noarch')}
     packageNames_new |= matchedpackages if len(matchedpackages) > 0 else {packageName}
   packageNames = list(packageNames_new)
   allPackageNames = set(packageNames)
@@ -226,24 +258,41 @@ def packagesDownload(packageNames, withDependencies = False, srcpkg = False, noc
   packageFilenames = []
   while len(packageNames) > 0:
     packName = packageNames.pop()
-    package = _findPackage(packName, srcpkg)
-    if package == None:
+    package = _findPackage(packName, project, srcpkg)
+    if package is None:
       logging.error('Package %s not found', packName)
-      continue
+      alt_packages = search_packages(packName, srcpkg)
+      if len(alt_packages) > 0:
+          logging.error('Did you mean:')
+          for alt_pkg in alt_packages:
+              logging.error('\t- {}'.format(re.sub('^mingw(32|64)-', '', alt_pkg)))
+      logging.error('Exiting without installing.')
+      sys.exit(os.EX_NOINPUT)
     dependencies = _checkPackageRequirements(package, allPackageNames)
     if withDependencies and len(dependencies) > 0:
       packageNames.extend(dependencies)
       allPackageNames |= dependencies
-    if packName[-6:] == '-devel':
+    if packName[-6:] == '-devel' and _findPackage(packName[:-6], project, srcpkg) is not None:
         logging.warning('{} is a devel package. Adding {}'.format(packName, packName[:-6]))
         packageNames.append(packName[:-6])
         allPackageNames.add(packName[:-6])
     localFilenameFull = os.path.join(_packageCacheDirectory, package['filename'])
+    # First removing any outdated version of the rpm.
+    package_basename = packageBaseName(package['filename'])
+    for f in os.listdir(_packageCacheDirectory):
+        if packageBaseName(f) == package_basename and f != package['filename']:
+            logging.warning('Deleting outdated cached version of {}.'.format(package_basename))
+            os.unlink(os.path.join(_packageCacheDirectory, f))
     if nocache or not os.path.exists(localFilenameFull):
-      logging.warning('Downloading %s', package['filename'])
-      urlretrieve(package['url'], localFilenameFull)
+        logging.warning('Downloading %s', package['filename'])
+        # When I download a rpm, I would also remove any extracted cpio
+        # of this package which may have been left behind.
+        for f in os.listdir(_extractedCacheDirectory):
+            if packageBaseName(f) == package_basename:
+                os.unlink(os.path.join(_extractedCacheDirectory, f))
+        urlretrieve(package['url'], localFilenameFull)
     else:
-      logging.warning('Using cached package %s', localFilenameFull)
+        logging.warning('Using cached package %s', localFilenameFull)
     packageFilenames.append(package['filename'])
   return (allPackageNames, packageFilenames)
 
@@ -284,19 +333,24 @@ def packagesExtract(packageFilenames, srcpkg=False):
   for packageFilename in packageFilenames :
     logging.warning('Extracting %s', packageFilename)
     rpm_path = os.path.join(_packageCacheDirectory, packageFilename)
-    cpioFilename = os.path.join(_extractedCacheDirectory, os.path.splitext(packageFilename)[0] + '.cpio')
-    if shutil.which('rpm2cpio') is None and not os.path.exists(cpioFilename):
-      # If using 7z, we have to make an intermediary step.
-      if not _extractFile(rpm_path, _extractedCacheDirectory):
-        return False
-      if srcpkg:
-        return _extractFile(cpioFilename, os.path.join(_extractedFilesDirectory, os.path.splitext(packageFilename)[0]))
-      else:
-        return _extractFile(cpioFilename, _extractedFilesDirectory)
+    if shutil.which('rpm2cpio') is None:
+        # If using 7z, we have to make an intermediary step.
+        cpioFilename = os.path.join(_extractedCacheDirectory, os.path.splitext(packageFilename)[0] + '.cpio')
+        if not os.path.exists(cpioFilename) and not _extractFile(rpm_path, _extractedCacheDirectory):
+            return False
+        if srcpkg:
+          if not _extractFile(cpioFilename, os.path.join(_extractedFilesDirectory, os.path.splitext(packageFilename)[0])):
+              return False
+        else:
+          if not _extractFile(cpioFilename, _extractedFilesDirectory):
+              return False
     elif srcpkg:
-      return _extractFile(rpm_path, os.path.join(_extractedFilesDirectory, os.path.splitext(packageFilename)[0]))
+        if not _extractFile(rpm_path, os.path.join(_extractedFilesDirectory, os.path.splitext(packageFilename)[0])):
+            return False
     else:
-      return _extractFile(rpm_path, _extractedFilesDirectory)
+        if not _extractFile(rpm_path, _extractedFilesDirectory):
+            return False
+  return True
 
 def move_files(from_file, to_file):
     if os.path.isdir(from_file):
@@ -387,11 +441,23 @@ def SetExecutableBit():
     bin_dir = os.path.join(prefix, 'bin')
     if os.path.isdir(bin_dir):
         for f in os.listdir(bin_dir):
-            os.chmod(os.path.join(bin_dir, f), 0o755)
+            # Make sure I chmod only binary in prefix/ and not linked from elsewhere
+            # because I have only control on prefix for sure.
+            fullpath = os.path.join(bin_dir, f)
+            if os.path.islink (fullpath):
+                link_path = os.path.abspath (os.readlink (fullpath))
+                if link_path.find (os.path.abspath (prefix)) != 0:
+                    continue
+            os.chmod(fullpath, 0o755)
     # set executable bit on libraries and executables whatever the path.
     for root, dirs, files in os.walk(prefix):
         for filename in {f for f in files if f.endswith('.dll') or f.endswith('.exe')} | set(dirs):
-            os.chmod(os.path.join(root, filename), 0o755)
+            fullpath = os.path.join(root, filename)
+            if os.path.islink (fullpath):
+                link_path = os.path.abspath (os.readlink (fullpath))
+                if link_path.find (os.path.abspath (prefix)) != 0:
+                    continue
+            os.chmod(fullpath, 0o755)
 
 def GetOptions():
   from optparse import OptionParser, OptionGroup #3.2: use argparse
@@ -401,7 +467,7 @@ def GetOptions():
 
   # Options specifiying download repository
   default_project = "windows:mingw:win32"
-  default_repository = "openSUSE_11.4"
+  default_repository = "openSUSE_13.1"
   default_repo_url = "http://download.opensuse.org/repositories/PROJECT/REPOSITORY/"
   repoOptions = OptionGroup(parser, "Specify download repository")
   repoOptions.add_option("-p", "--project", dest="project", default=default_project,
@@ -421,6 +487,7 @@ def GetOptions():
   packageOptions.add_option("--nocache", action="store_true", dest="nocache", default=False,
                             help="Force package download even if it is in cache.")
   packageOptions.add_option("--list-files", action="store_true", dest="list_files", default=False, help="Only list the files of a package")
+  packageOptions.add_option("--search", action="store_true", dest="search", default=False, help="Search packages.")
   packageOptions.add_option("--info", action="store_true", dest="info", default=False, help="Output information about a package")
   packageOptions.add_option("--uninstall", action="store_true", dest="uninstall", default=False, help="Uninstall the list of packages")
   parser.add_option_group(packageOptions)
@@ -459,6 +526,30 @@ if __name__ == "__main__":
   except Exception as e:
     sys.exit('Error opening repository:\n\t%s\n\t%s' % (repository, e))
 
+  if options.search:
+    if (len(packages) == 0):
+        logging.error('Please provide at least one package.\n')
+        sys.exit(os.EX_USAGE)
+    if options.srcpkg:
+        package_type = 'Source package'
+    else:
+        package_type = 'Package'
+    for keyword in packages:
+        alt_packages = search_packages(keyword, options.srcpkg)
+        if len(alt_packages) > 0:
+            sys.stdout.write('The following packages were found for the search "{}":\n'.format(keyword))
+            for alt_pkg in alt_packages:
+                sys.stdout.write('\t- {}\n'.format(re.sub('^mingw(32|64)-', '', alt_pkg)))
+        else:
+            sys.stdout.write('"{}" not found in any package name.\n'.format(keyword))
+        if options.list_files:
+            alt_packages = search_packages(keyword, options.srcpkg, options.list_files)
+            if len(alt_packages) > 0:
+                sys.stdout.write('The following packages have files matching the search "{}":\n'.format(keyword))
+                for alt_pkg in alt_packages:
+                    sys.stdout.write('\t- {}\n'.format(re.sub('^mingw(32|64)-', '', alt_pkg)))
+    sys.exit(os.EX_OK)
+
   if options.list_files:
     if (len(packages) == 0):
         logging.error('Please provide at least one package.\n')
@@ -490,15 +581,15 @@ if __name__ == "__main__":
     else:
         package_type = 'Package'
     for pkg in packages:
-        package = _findPackage(pkg, options.srcpkg)
+        package = _findPackage(pkg, options.project, options.srcpkg)
         if package is None:
-            if options.project == 'windows:mingw:win64':
-                package = _findPackage("mingw64-" + pkg, options.srcpkg)
-            elif options.project == 'windows:mingw:win32':
-                package = _findPackage("mingw32-" + pkg, options.srcpkg)
-            if package is None:
-                sys.stderr.write('{} "{}" unknown.\n'.format(package_type, pkg))
-                continue
+            sys.stderr.write('{} "{}" unknown.\n'.format(package_type, pkg))
+            alt_packages = search_packages(pkg, options.srcpkg)
+            if len(alt_packages) > 0:
+                logging.error('\tDid you mean:')
+                for alt_pkg in alt_packages:
+                    logging.error('\t- {}'.format(re.sub('^mingw(32|64)-', '', alt_pkg)))
+            continue
         sys.stdout.write('{} "{}":\n'.format(package_type, package['name']))
         sys.stdout.write('\tSummary: {}\n'.format(package['summary']))
         sys.stdout.write('\tProject URL: {}\n'.format(package['project_url']))
@@ -546,6 +637,13 @@ if __name__ == "__main__":
                     except FileNotFoundError:
                         # Let's just ignore already removed files.
                         pass
+            # Finally I deleted any cached rpm and cpio.
+            for f in os.listdir(_packageCacheDirectory):
+                if f[:len(real_name)] == real_name:
+                    os.unlink(os.path.join(_packageCacheDirectory, f))
+            for f in os.listdir(_extractedCacheDirectory):
+                if f[:len(real_name)] == real_name:
+                    os.unlink(os.path.join(_extractedCacheDirectory, f))
     sys.exit(os.EX_OK)
 
   if options.clean:
@@ -564,26 +662,36 @@ if __name__ == "__main__":
     sys.exit(os.EX_CANTCREAT)
 
   if options.makezip or options.metadata:
-    package = _findPackage(args[0]) #if args[0].endswith('.rpm') 
-    if package == None: package = _findPackage("mingw32-"+args[0], options.srcpkg)
-    if package == None: package = _findPackage("mingw64-"+args[0], options.srcpkg)
+    package = _findPackage(args[0], options.project, options.srcpkg)
     if package == None:
-      sys.exit('Package not found:\n\t%s' % args[0])
+      logging.error('Package not found:\n\t%s' % args[0])
+      alt_packages = search_packages(args[0], options.srcpkg)
+      if len(alt_packages) > 0:
+          logging.error('Did you mean:')
+          for alt_pkg in alt_packages:
+              logging.error('\t- {}'.format(re.sub('^mingw(32|64)-', '', alt_pkg)))
+      sys.exit(os.EX_UNAVAILABLE)
     packageBasename = re.sub('^mingw(32|64)-|\\.noarch|\\.rpm$', '', package['filename'])
 
-  (packages, packages_rpm) = packagesDownload(packages, options.withdeps, options.srcpkg, options.nocache)
+  (packages, packages_rpm) = packagesDownload(packages, options.project, options.withdeps, options.srcpkg, options.nocache)
 
   if not packagesExtract(packages_rpm, options.srcpkg):
     logging.error('A package failed to extract. Please report a bug.')
     sys.exit(os.EX_CANTCREAT)
 
   extracted_prefix = GetBaseDirectory(options.project)
+  sys.stdout.write('Installing...\n')
+  sys.stdout.flush()
   move_files(extracted_prefix, prefix)
-  logging.warning ('Fixing symlinks')
+  sys.stdout.write('Fixing symlinks...\n')
   sys.stdout.flush()
   for package in packages:
       fix_package_symlinks (package, options)
+  sys.stdout.write('Make binaries executable...\n')
+  sys.stdout.flush()
   SetExecutableBit()
+  sys.stdout.write('All installations done!\n')
+  sys.stdout.flush()
 
   if options.metadata:
     cleanup = lambda n: re.sub('^mingw(?:32|64)-(.*)', '\\1', re.sub('^mingw(?:32|64)[(](.*)[)]', '\\1', n))
